@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // Basic Usage Environment: for a simple, non-scripted, console application
 // Implementation
 
@@ -32,7 +32,7 @@ BasicUsageEnvironment::BasicUsageEnvironment(TaskScheduler& taskScheduler)
   if (!initializeWinsockIfNecessary()) {
     setResultErrMsg("Failed to initialize 'winsock': ");
     reportBackgroundError();
-    abort();
+    internalError();
   }
 #endif
 }
@@ -54,26 +54,27 @@ int BasicUsageEnvironment::getErrno() const {
 }
 
 UsageEnvironment& BasicUsageEnvironment::operator<<(char const* str) {
-	fprintf(stderr, "%s", str);
-	return *this;
+  if (str == NULL) str = "(NULL)"; // sanity check
+  fprintf(stderr, "%s", str);
+  return *this;
 }
 
 UsageEnvironment& BasicUsageEnvironment::operator<<(int i) {
-	fprintf(stderr, "%d", i);
-	return *this;
+  fprintf(stderr, "%d", i);
+  return *this;
 }
 
 UsageEnvironment& BasicUsageEnvironment::operator<<(unsigned u) {
-	fprintf(stderr, "%u", u);
-	return *this;
+  fprintf(stderr, "%u", u);
+  return *this;
 }
 
 UsageEnvironment& BasicUsageEnvironment::operator<<(double d) {
-	fprintf(stderr, "%f", d);
-	return *this;
+  fprintf(stderr, "%f", d);
+  return *this;
 }
 
 UsageEnvironment& BasicUsageEnvironment::operator<<(void* p) {
-	fprintf(stderr, "%p", p);
-	return *this;
+  fprintf(stderr, "%p", p);
+  return *this;
 }
