@@ -13,13 +13,12 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // A C++ equivalent to the standard C routine "strdup()".
 // This generates a char* that can be deleted using "delete[]"
 // Implementation
 
 #include "strDup.hh"
-#include "string.h"
 
 char* strDup(char const* str) {
   if (str == NULL) return NULL;
@@ -33,10 +32,19 @@ char* strDup(char const* str) {
 }
 
 char* strDupSize(char const* str) {
-  if (str == NULL) return NULL;
-  size_t len = strlen(str) + 1;
-  char* copy = new char[len];
+  size_t dummy;
+
+  return strDupSize(str, dummy);
+}
+
+char* strDupSize(char const* str, size_t& resultBufSize) {
+  if (str == NULL) {
+    resultBufSize = 0;
+    return NULL;
+  }
+
+  resultBufSize = strlen(str) + 1;
+  char* copy = new char[resultBufSize];
 
   return copy;
 }
-
