@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2009 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // Framed Filters
 // Implementation
 
@@ -23,6 +23,13 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 ////////// FramedFilter //////////
 #include <string.h>
 
+void FramedFilter::detachInputSource() {
+  if (fInputSource != NULL) {
+    fInputSource->stopGettingFrames();
+    reassignInputSource(NULL);
+  }
+}
+
 FramedFilter::FramedFilter(UsageEnvironment& env,
 			   FramedSource* inputSource)
   : FramedSource(env),
@@ -47,5 +54,6 @@ void FramedFilter::getAttributes() const {
 }
 
 void FramedFilter::doStopGettingFrames() {
+  FramedSource::doStopGettingFrames();
   if (fInputSource != NULL) fInputSource->stopGettingFrames();
 }
