@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // Author Bernhard Feiten
 // A filter that breaks up an H.263plus video stream into frames.
 //
@@ -61,12 +61,6 @@ H263plusVideoStreamFramer::~H263plusVideoStreamFramer()
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-#ifdef DEBUG
-static struct timeval firstPT;
-#endif
-
-
 ///////////////////////////////////////////////////////////////////////////////
 void H263plusVideoStreamFramer::doGetNextFrame()
 {
