@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // H.264 Video RTP Sources
 // Implementation
 
@@ -67,51 +67,48 @@ Boolean H264VideoRTPSource
                        unsigned& resultSpecialHeaderSize) {
   unsigned char* headerStart = packet->data();
   unsigned packetSize = packet->dataSize();
+  unsigned numBytesToSkip;
 
-  // The header has a minimum size of 0, since the NAL header is used
-  // as a payload header
-  unsigned expectedHeaderSize = 0;
-
-  // Check if the type field is 28 (FU-A) or 29 (FU-B)
+  // Check the 'nal_unit_type' for special 'aggregation' or 'fragmentation' packets:
+  if (packetSize < 1) return False;
   fCurPacketNALUnitType = (headerStart[0]&0x1F);
   switch (fCurPacketNALUnitType) {
   case 24: { // STAP-A
-    expectedHeaderSize = 1; // discard the type byte
+    numBytesToSkip = 1; // discard the type byte
     break;
   }
   case 25: case 26: case 27: { // STAP-B, MTAP16, or MTAP24
-    expectedHeaderSize = 3; // discard the type byte, and the initial DON
+    numBytesToSkip = 3; // discard the type byte, and the initial DON
     break;
   }
   case 28: case 29: { // // FU-A or FU-B
     // For these NALUs, the first two bytes are the FU indicator and the FU header.
-    // If the start bit is set, we reconstruct the original NAL header:
+    // If the start bit is set, we reconstruct the original NAL header into byte 1:
+    if (packetSize < 2) return False;
     unsigned char startBit = headerStart[1]&0x80;
     unsigned char endBit = headerStart[1]&0x40;
     if (startBit) {
-      expectedHeaderSize = 1;
-      if (packetSize < expectedHeaderSize) return False;
-
-      headerStart[1] = (headerStart[0]&0xE0)+(headerStart[1]&0x1F);
       fCurrentPacketBeginsFrame = True;
+
+      headerStart[1] = (headerStart[0]&0xE0)|(headerStart[1]&0x1F);
+      numBytesToSkip = 1;
     } else {
-      // If the startbit is not set, both the FU indicator and header
-      // can be discarded
-      expectedHeaderSize = 2;
-      if (packetSize < expectedHeaderSize) return False;
+      // The start bit is not set, so we skip both the FU indicator and header:
       fCurrentPacketBeginsFrame = False;
+      numBytesToSkip = 2;
     }
     fCurrentPacketCompletesFrame = (endBit != 0);
     break;
   }
   default: {
-    // This packet contains one or more complete, decodable NAL units
+    // This packet contains one complete NAL unit:
     fCurrentPacketBeginsFrame = fCurrentPacketCompletesFrame = True;
+    numBytesToSkip = 0;
     break;
   }
   }
 
-  resultSpecialHeaderSize = expectedHeaderSize;
+  resultSpecialHeaderSize = numBytesToSkip;
   return True;
 }
 
