@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // MPEG-1 or MPEG-2 Video RTP Sources
 // Implementation
 
@@ -52,7 +52,7 @@ Boolean MPEG1or2VideoRTPSource
   u_int32_t eBit = header&0x00000800; // end-of-slice
 
   fCurrentPacketBeginsFrame = (sBit|bBit) != 0;
-  fCurrentPacketCompletesFrame = ((sBit&~bBit)|eBit) != 0;
+  fCurrentPacketCompletesFrame = ((sBit != 0) && (bBit == 0)) || (eBit != 0);
 
   resultSpecialHeaderSize = 4;
   return True;
