@@ -31,10 +31,11 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 class ProxyServerMediaSubsession: public OnDemandServerMediaSubsession {
 public:
-  ProxyServerMediaSubsession(MediaSubsession& mediaSubsession);
+  ProxyServerMediaSubsession(MediaSubsession& mediaSubsession,
+			     portNumBits initialPortNum, Boolean multiplexRTCPWithRTP);
   virtual ~ProxyServerMediaSubsession();
 
-  char const* codecName() const { return fClientMediaSubsession.codecName(); }
+  char const* codecName() const { return fCodecName; }
 
 private: // redefined virtual functions
   virtual FramedSource* createNewStreamSource(unsigned clientSessionId,
@@ -43,6 +44,9 @@ private: // redefined virtual functions
   virtual RTPSink* createNewRTPSink(Groupsock* rtpGroupsock,
                                     unsigned char rtpPayloadTypeIfDynamic,
                                     FramedSource* inputSource);
+  virtual Groupsock* createGroupsock(struct in_addr const& addr, Port port);
+  virtual RTCPInstance* createRTCP(Groupsock* RTCPgs, unsigned totSessionBW, /* in kbps */
+				   unsigned char const* cname, RTPSink* sink);
 
 private:
   static void subsessionByeHandler(void* clientData);
@@ -53,6 +57,7 @@ private:
 private:
   friend class ProxyRTSPClient;
   MediaSubsession& fClientMediaSubsession; // the 'client' media subsession object that corresponds to this 'server' media subsession
+  char const* fCodecName;  // copied from "fClientMediaSubsession" once it's been set up
   ProxyServerMediaSubsession* fNext; // used when we're part of a queue
   Boolean fHaveSetupStream;
 };
@@ -75,27 +80,33 @@ defaultCreateNewProxyRTSPClientFunc(ProxyServerMediaSession& ourServerMediaSessi
 }
 
 ProxyServerMediaSession* ProxyServerMediaSession
-::createNew(UsageEnvironment& env, RTSPServer* ourRTSPServer,
+::createNew(UsageEnvironment& env, GenericMediaServer* ourMediaServer,
 	    char const* inputStreamURL, char const* streamName,
 	    char const* username, char const* password,
-	    portNumBits tunnelOverHTTPPortNum, int verbosityLevel, int socketNumToServer) {
-  return new ProxyServerMediaSession(env, ourRTSPServer, inputStreamURL, streamName, username, password,
-				     tunnelOverHTTPPortNum, verbosityLevel, socketNumToServer);
+	    portNumBits tunnelOverHTTPPortNum, int verbosityLevel, int socketNumToServer,
+	    MediaTranscodingTable* transcodingTable) {
+  return new ProxyServerMediaSession(env, ourMediaServer, inputStreamURL, streamName, username, password,
+				     tunnelOverHTTPPortNum, verbosityLevel, socketNumToServer,
+				     transcodingTable);
 }
 
 
 ProxyServerMediaSession
-::ProxyServerMediaSession(UsageEnvironment& env, RTSPServer* ourRTSPServer,
+::ProxyServerMediaSession(UsageEnvironment& env, GenericMediaServer* ourMediaServer,
 			  char const* inputStreamURL, char const* streamName,
 			  char const* username, char const* password,
 			  portNumBits tunnelOverHTTPPortNum, int verbosityLevel,
 			  int socketNumToServer,
-			  createNewProxyRTSPClientFunc* ourCreateNewProxyRTSPClientFunc)
+			  MediaTranscodingTable* transcodingTable,
+			  createNewProxyRTSPClientFunc* ourCreateNewProxyRTSPClientFunc,
+			  portNumBits initialPortNum, Boolean multiplexRTCPWithRTP)
   : ServerMediaSession(env, streamName, NULL, NULL, False, NULL),
-    describeCompletedFlag(0), fOurRTSPServer(ourRTSPServer), fClientMediaSession(NULL),
+    describeCompletedFlag(0), fOurMediaServer(ourMediaServer), fClientMediaSession(NULL),
     fVerbosityLevel(verbosityLevel),
     fPresentationTimeSessionNormalizer(new PresentationTimeSessionNormalizer(envir())),
-    fCreateNewProxyRTSPClientFunc(ourCreateNewProxyRTSPClientFunc) {
+    fCreateNewProxyRTSPClientFunc(ourCreateNewProxyRTSPClientFunc),
+    fTranscodingTable(transcodingTable),
+    fInitialPortNum(initialPortNum), fMultiplexRTCPWithRTP(multiplexRTCPWithRTP) {
   // Open a RTSP connection to the input stream, and send a "DESCRIBE" command.
   // We'll use the SDP description in the response to set ourselves up.
   fProxyRTSPClient
@@ -112,18 +123,33 @@ ProxyServerMediaSession::~ProxyServerMediaSession() {
   }
 
   // Begin by sending a "TEARDOWN" command (without checking for a response):
-  if (fProxyRTSPClient != NULL) fProxyRTSPClient->sendTeardownCommand(*fClientMediaSession, NULL, fProxyRTSPClient->auth());
+  if (fProxyRTSPClient != NULL && fClientMediaSession != NULL) {
+    fProxyRTSPClient->sendTeardownCommand(*fClientMediaSession, NULL, fProxyRTSPClient->auth());
+  }
 
   // Then delete our state:
+  Medium::close(fTranscodingTable);
   Medium::close(fClientMediaSession);
   Medium::close(fProxyRTSPClient);
-  delete fPresentationTimeSessionNormalizer;
+  Medium::close(fPresentationTimeSessionNormalizer);
 }
 
 char const* ProxyServerMediaSession::url() const {
   return fProxyRTSPClient == NULL ? NULL : fProxyRTSPClient->url();
 }
 
+Groupsock* ProxyServerMediaSession::createGroupsock(struct in_addr const& addr, Port port) {
+  // Default implementation; may be redefined by subclasses:
+  return new Groupsock(envir(), addr, port, 255);
+}
+
+RTCPInstance* ProxyServerMediaSession
+::createRTCP(Groupsock* RTCPgs, unsigned totSessionBW, /* in kbps */
+	     unsigned char const* cname, RTPSink* sink) {
+  // Default implementation; may be redefined by subclasses:
+  return RTCPInstance::createNew(envir(), RTCPgs, totSessionBW, cname, sink, NULL/*we're a server*/);
+}
+
 void ProxyServerMediaSession::continueAfterDESCRIBE(char const* sdpDescription) {
   describeCompletedFlag = 1;
 
@@ -135,7 +161,8 @@ void ProxyServerMediaSession::continueAfterDESCRIBE(char const* sdpDescription)
 
     MediaSubsessionIterator iter(*fClientMediaSession);
     for (MediaSubsession* mss = iter.next(); mss != NULL; mss = iter.next()) {
-      ServerMediaSubsession* smss = new ProxyServerMediaSubsession(*mss);
+      ServerMediaSubsession* smss
+	= new ProxyServerMediaSubsession(*mss, fInitialPortNum, fMultiplexRTCPWithRTP);
       addSubsession(smss);
       if (fVerbosityLevel > 0) {
 	envir() << *this << " added new \"ProxyServerMediaSubsession\" for "
@@ -147,9 +174,9 @@ void ProxyServerMediaSession::continueAfterDESCRIBE(char const* sdpDescription)
 
 void ProxyServerMediaSession::resetDESCRIBEState() {
   // Delete all of our "ProxyServerMediaSubsession"s; they'll get set up again once we get a response to the new "DESCRIBE".
-  if (fOurRTSPServer != NULL) {
-    // First, close any RTSP client connections that may have already been set up:
-    fOurRTSPServer->closeAllClientSessionsForServerMediaSession(this);
+  if (fOurMediaServer != NULL) {
+    // First, close any client connections that may have already been set up:
+    fOurMediaServer->closeAllClientSessionsForServerMediaSession(this);
   }
   deleteAllSubsessions();
 
@@ -174,9 +201,12 @@ static void continueAfterDESCRIBE(RTSPClient* rtspClient, int resultCode, char*
 }
 
 static void continueAfterSETUP(RTSPClient* rtspClient, int resultCode, char* resultString) {
-  if (resultCode == 0) {
-    ((ProxyRTSPClient*)rtspClient)->continueAfterSETUP();
-  }
+  ((ProxyRTSPClient*)rtspClient)->continueAfterSETUP(resultCode);
+  delete[] resultString;
+}
+
+static void continueAfterPLAY(RTSPClient* rtspClient, int resultCode, char* resultString) {
+  ((ProxyRTSPClient*)rtspClient)->continueAfterPLAY(resultCode);
   delete[] resultString;
 }
 
@@ -290,12 +320,19 @@ void ProxyRTSPClient::continueAfterLivenessCommand(int resultCode, Boolean serve
 
 #define SUBSESSION_TIMEOUT_SECONDS 10 // how many seconds to wait for the last track's "SETUP" to be done (note below)
 
-void ProxyRTSPClient::continueAfterSETUP() {
+void ProxyRTSPClient::continueAfterSETUP(int resultCode) {
+  if (resultCode != 0) {
+    // The "SETUP" command failed, so reset the connection with the 'back-end' server and start over,
+    // just as if a periodic 'liveness' check with the 'back-end' server had failed:
+    continueAfterLivenessCommand(resultCode, fServerSupportsGetParameter);
+    return;
+  }
+
   if (fVerbosityLevel > 0) {
-    envir() << *this << "::continueAfterSETUP(): head codec: " << fSetupQueueHead->fClientMediaSubsession.codecName()
+    envir() << *this << "::continueAfterSETUP(): head codec: " << fSetupQueueHead->codecName()
 	    << "; numSubsessions " << fSetupQueueHead->fParentSession->numSubsessions() << "\n\tqueue:";
     for (ProxyServerMediaSubsession* p = fSetupQueueHead; p != NULL; p = p->fNext) {
-      envir() << "\t" << p->fClientMediaSubsession.codecName();
+      envir() << "\t" << p->codecName();
     }
     envir() << "\n";
   }
@@ -317,7 +354,7 @@ void ProxyRTSPClient::continueAfterSETUP() {
     if (fNumSetupsDone >= smss->fParentSession->numSubsessions()) {
       // We've now finished setting up each of our subsessions (i.e., 'tracks').
       // Continue by sending a "PLAY" command (an 'aggregate' "PLAY" command, on the whole session):
-      sendPlayCommand(smss->fClientMediaSubsession.parentSession(), NULL, -1.0f, -1.0f, 1.0f, fOurAuthenticator);
+      sendPlayCommand(smss->fClientMediaSubsession.parentSession(), ::continueAfterPLAY, -1.0f, -1.0f, 1.0f, fOurAuthenticator);
           // the "-1.0f" "start" parameter causes the "PLAY" to be sent without a "Range:" header, in case we'd already done
           // a "PLAY" before (as a result of a 'subsession timeout' (note below))
       fLastCommandWasPLAY = True;
@@ -332,6 +369,14 @@ void ProxyRTSPClient::continueAfterSETUP() {
   }
 }
 
+void ProxyRTSPClient::continueAfterPLAY(int resultCode) {
+  if (resultCode != 0) {
+    // The "PLAY" command failed, so reset the connection with the 'back-end' server and start over,
+    // just as if a periodic 'liveness' check with the 'back-end' server had failed:
+    continueAfterLivenessCommand(resultCode, fServerSupportsGetParameter);
+  }
+}
+
 void ProxyRTSPClient::scheduleLivenessCommand() {
   // Delay a random time before sending another 'liveness' command.
   unsigned delayMax = sessionTimeoutParameter(); // if the server specified a maximum time between 'liveness' probes, then use that
@@ -398,16 +443,20 @@ void ProxyRTSPClient::subsessionTimeout(void* clientData) {
 void ProxyRTSPClient::handleSubsessionTimeout() {
   // We still have one or more subsessions ('tracks') left to "SETUP".  But we can't wait any longer for them.  Send a "PLAY" now:
   MediaSession* sess = fOurServerMediaSession.fClientMediaSession;
-  if (sess != NULL) sendPlayCommand(*sess, NULL, -1.0f, -1.0f, 1.0f, fOurAuthenticator);
+  if (sess != NULL) sendPlayCommand(*sess, ::continueAfterPLAY, -1.0f, -1.0f, 1.0f, fOurAuthenticator);
   fLastCommandWasPLAY = True;
 }
 
 
 //////// "ProxyServerMediaSubsession" implementation //////////
 
-ProxyServerMediaSubsession::ProxyServerMediaSubsession(MediaSubsession& mediaSubsession)
-  : OnDemandServerMediaSubsession(mediaSubsession.parentSession().envir(), True/*reuseFirstSource*/),
-    fClientMediaSubsession(mediaSubsession), fNext(NULL), fHaveSetupStream(False) {
+ProxyServerMediaSubsession
+::ProxyServerMediaSubsession(MediaSubsession& mediaSubsession,
+			     portNumBits initialPortNum, Boolean multiplexRTCPWithRTP)
+  : OnDemandServerMediaSubsession(mediaSubsession.parentSession().envir(), True/*reuseFirstSource*/,
+				  initialPortNum, multiplexRTCPWithRTP),
+    fClientMediaSubsession(mediaSubsession), fCodecName(strDup(mediaSubsession.codecName())),
+    fNext(NULL), fHaveSetupStream(False) {
 }
 
 UsageEnvironment& operator<<(UsageEnvironment& env, const ProxyServerMediaSubsession& psmss) { // used for debugging
@@ -418,6 +467,8 @@ ProxyServerMediaSubsession::~ProxyServerMediaSubsession() {
   if (verbosityLevel() > 0) {
     envir() << *this << "::~ProxyServerMediaSubsession()\n";
   }
+
+  delete[] (char*)fCodecName;
 }
 
 FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientSessionId, unsigned& estBitrate) {
@@ -437,31 +488,42 @@ FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientS
     }
 
     if (fClientMediaSubsession.readSource() != NULL) {
-      // Add to the front of all data sources a filter that will 'normalize' their frames' presentation times,
-      // before the frames get re-transmitted by our server:
-      char const* const codecName = fClientMediaSubsession.codecName();
+      // First, check whether we have defined a 'transcoder' filter to be used with this codec:
+      if (sms->fTranscodingTable != NULL) {
+	char* outputCodecName;
+	FramedFilter* transcoder
+	  = sms->fTranscodingTable->lookupTranscoder(fClientMediaSubsession, outputCodecName);
+	if (transcoder != NULL) {
+	  fClientMediaSubsession.addFilter(transcoder);
+	  delete[] (char*)fCodecName; fCodecName = outputCodecName;
+	}
+      }
+
+      // Then, add to the front of all data sources a filter that will 'normalize' their frames'
+      // presentation times, before the frames get re-transmitted by our server:
       FramedFilter* normalizerFilter = sms->fPresentationTimeSessionNormalizer
-	->createNewPresentationTimeSubsessionNormalizer(fClientMediaSubsession.readSource(), fClientMediaSubsession.rtpSource(),
-							codecName);
+	->createNewPresentationTimeSubsessionNormalizer(fClientMediaSubsession.readSource(),
+							fClientMediaSubsession.rtpSource(),
+							fCodecName);
       fClientMediaSubsession.addFilter(normalizerFilter);
 
       // Some data sources require a 'framer' object to be added, before they can be fed into
       // a "RTPSink".  Adjust for this now:
-      if (strcmp(codecName, "H264") == 0) {
+      if (strcmp(fCodecName, "H264") == 0) {
 	fClientMediaSubsession.addFilter(H264VideoStreamDiscreteFramer
 					 ::createNew(envir(), fClientMediaSubsession.readSource()));
-      } else if (strcmp(codecName, "H265") == 0) {
+      } else if (strcmp(fCodecName, "H265") == 0) {
 	fClientMediaSubsession.addFilter(H265VideoStreamDiscreteFramer
 					 ::createNew(envir(), fClientMediaSubsession.readSource()));
-      } else if (strcmp(codecName, "MP4V-ES") == 0) {
+      } else if (strcmp(fCodecName, "MP4V-ES") == 0) {
 	fClientMediaSubsession.addFilter(MPEG4VideoStreamDiscreteFramer
 					 ::createNew(envir(), fClientMediaSubsession.readSource(),
 						     True/* leave PTs unmodified*/));
-      } else if (strcmp(codecName, "MPV") == 0) {
+      } else if (strcmp(fCodecName, "MPV") == 0) {
 	fClientMediaSubsession.addFilter(MPEG1or2VideoStreamDiscreteFramer
 					 ::createNew(envir(), fClientMediaSubsession.readSource(),
 						     False, 5.0, True/* leave PTs unmodified*/));
-      } else if (strcmp(codecName, "DV") == 0) {
+      } else if (strcmp(fCodecName, "DV") == 0) {
 	fClientMediaSubsession.addFilter(DVVideoStreamFramer
 					 ::createNew(envir(), fClientMediaSubsession.readSource(),
 						     False, True/* leave PTs unmodified*/));
@@ -501,7 +563,7 @@ FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientS
       // have been called here), so we know that the substream was previously "PAUSE"d.  Send "PLAY" downstream once again,
       // to resume the stream:
       if (!proxyRTSPClient->fLastCommandWasPLAY) { // so that we send only one "PLAY"; not one for each subsession
-	proxyRTSPClient->sendPlayCommand(fClientMediaSubsession.parentSession(), NULL, -1.0f/*resume from previous point*/,
+	proxyRTSPClient->sendPlayCommand(fClientMediaSubsession.parentSession(), ::continueAfterPLAY, -1.0f/*resume from previous point*/,
 					 -1.0f, 1.0f, proxyRTSPClient->auth());
 	proxyRTSPClient->fLastCommandWasPLAY = True;
       }
@@ -538,85 +600,85 @@ RTPSink* ProxyServerMediaSubsession
   }
 
   // Create (and return) the appropriate "RTPSink" object for our codec:
+  // (Note: The configuration string might not be correct if a transcoder is used. FIX!) #####
   RTPSink* newSink;
-  char const* const codecName = fClientMediaSubsession.codecName();
-  if (strcmp(codecName, "AC3") == 0 || strcmp(codecName, "EAC3") == 0) {
+  if (strcmp(fCodecName, "AC3") == 0 || strcmp(fCodecName, "EAC3") == 0) {
     newSink = AC3AudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					 fClientMediaSubsession.rtpTimestampFrequency()); 
 #if 0 // This code does not work; do *not* enable it:
-  } else if (strcmp(codecName, "AMR") == 0 || strcmp(codecName, "AMR-WB") == 0) {
-    Boolean isWideband = strcmp(codecName, "AMR-WB") == 0;
+  } else if (strcmp(fCodecName, "AMR") == 0 || strcmp(fCodecName, "AMR-WB") == 0) {
+    Boolean isWideband = strcmp(fCodecName, "AMR-WB") == 0;
     newSink = AMRAudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					 isWideband, fClientMediaSubsession.numChannels());
 #endif
-  } else if (strcmp(codecName, "DV") == 0) {
+  } else if (strcmp(fCodecName, "DV") == 0) {
     newSink = DVVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
-  } else if (strcmp(codecName, "GSM") == 0) {
+  } else if (strcmp(fCodecName, "GSM") == 0) {
     newSink = GSMAudioRTPSink::createNew(envir(), rtpGroupsock);
-  } else if (strcmp(codecName, "H263-1998") == 0 || strcmp(codecName, "H263-2000") == 0) {
+  } else if (strcmp(fCodecName, "H263-1998") == 0 || strcmp(fCodecName, "H263-2000") == 0) {
     newSink = H263plusVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					      fClientMediaSubsession.rtpTimestampFrequency()); 
-  } else if (strcmp(codecName, "H264") == 0) {
+  } else if (strcmp(fCodecName, "H264") == 0) {
     newSink = H264VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					  fClientMediaSubsession.fmtp_spropparametersets());
-  } else if (strcmp(codecName, "H265") == 0) {
+  } else if (strcmp(fCodecName, "H265") == 0) {
     newSink = H265VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					  fClientMediaSubsession.fmtp_spropvps(),
 					  fClientMediaSubsession.fmtp_spropsps(),
 					  fClientMediaSubsession.fmtp_sproppps());
-  } else if (strcmp(codecName, "JPEG") == 0) {
+  } else if (strcmp(fCodecName, "JPEG") == 0) {
     newSink = SimpleRTPSink::createNew(envir(), rtpGroupsock, 26, 90000, "video", "JPEG",
 				       1/*numChannels*/, False/*allowMultipleFramesPerPacket*/, False/*doNormalMBitRule*/);
-  } else if (strcmp(codecName, "MP4A-LATM") == 0) {
+  } else if (strcmp(fCodecName, "MP4A-LATM") == 0) {
     newSink = MPEG4LATMAudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					       fClientMediaSubsession.rtpTimestampFrequency(),
 					       fClientMediaSubsession.fmtp_config(),
 					       fClientMediaSubsession.numChannels());
-  } else if (strcmp(codecName, "MP4V-ES") == 0) {
+  } else if (strcmp(fCodecName, "MP4V-ES") == 0) {
     newSink = MPEG4ESVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					     fClientMediaSubsession.rtpTimestampFrequency(),
 					     fClientMediaSubsession.attrVal_unsigned("profile-level-id"),
 					     fClientMediaSubsession.fmtp_config()); 
-  } else if (strcmp(codecName, "MPA") == 0) {
+  } else if (strcmp(fCodecName, "MPA") == 0) {
     newSink = MPEG1or2AudioRTPSink::createNew(envir(), rtpGroupsock);
-  } else if (strcmp(codecName, "MPA-ROBUST") == 0) {
+  } else if (strcmp(fCodecName, "MPA-ROBUST") == 0) {
     newSink = MP3ADURTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
-  } else if (strcmp(codecName, "MPEG4-GENERIC") == 0) {
+  } else if (strcmp(fCodecName, "MPEG4-GENERIC") == 0) {
     newSink = MPEG4GenericRTPSink::createNew(envir(), rtpGroupsock,
 					     rtpPayloadTypeIfDynamic, fClientMediaSubsession.rtpTimestampFrequency(),
 					     fClientMediaSubsession.mediumName(),
 					     fClientMediaSubsession.attrVal_strToLower("mode"),
 					     fClientMediaSubsession.fmtp_config(), fClientMediaSubsession.numChannels());
-  } else if (strcmp(codecName, "MPV") == 0) {
+  } else if (strcmp(fCodecName, "MPV") == 0) {
     newSink = MPEG1or2VideoRTPSink::createNew(envir(), rtpGroupsock);
-  } else if (strcmp(codecName, "OPUS") == 0) {
+  } else if (strcmp(fCodecName, "OPUS") == 0) {
     newSink = SimpleRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 				       48000, "audio", "OPUS", 2, False/*only 1 Opus 'packet' in each RTP packet*/);
-  } else if (strcmp(codecName, "T140") == 0) {
+  } else if (strcmp(fCodecName, "T140") == 0) {
     newSink = T140TextRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
-  } else if (strcmp(codecName, "THEORA") == 0) {
+  } else if (strcmp(fCodecName, "THEORA") == 0) {
     newSink = TheoraVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					    fClientMediaSubsession.fmtp_config()); 
-  } else if (strcmp(codecName, "VORBIS") == 0) {
+  } else if (strcmp(fCodecName, "VORBIS") == 0) {
     newSink = VorbisAudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					    fClientMediaSubsession.rtpTimestampFrequency(), fClientMediaSubsession.numChannels(),
 					    fClientMediaSubsession.fmtp_config()); 
-  } else if (strcmp(codecName, "VP8") == 0) {
+  } else if (strcmp(fCodecName, "VP8") == 0) {
     newSink = VP8VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
-  } else if (strcmp(codecName, "VP9") == 0) {
+  } else if (strcmp(fCodecName, "VP9") == 0) {
     newSink = VP9VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
-  } else if (strcmp(codecName, "AMR") == 0 || strcmp(codecName, "AMR-WB") == 0) {
+  } else if (strcmp(fCodecName, "AMR") == 0 || strcmp(fCodecName, "AMR-WB") == 0) {
     // Proxying of these codecs is currently *not* supported, because the data received by the "RTPSource" object is not in a
     // form that can be fed directly into a corresponding "RTPSink" object.
     if (verbosityLevel() > 0) {
       envir() << "\treturns NULL (because we currently don't support the proxying of \""
-	      << fClientMediaSubsession.mediumName() << "/" << codecName << "\" streams)\n";
+	      << fClientMediaSubsession.mediumName() << "/" << fCodecName << "\" streams)\n";
     }
     return NULL;
-  } else if (strcmp(codecName, "QCELP") == 0 ||
-	     strcmp(codecName, "H261") == 0 ||
-	     strcmp(codecName, "H263-1998") == 0 || strcmp(codecName, "H263-2000") == 0 ||
-	     strcmp(codecName, "X-QT") == 0 || strcmp(codecName, "X-QUICKTIME") == 0) {
+  } else if (strcmp(fCodecName, "QCELP") == 0 ||
+	     strcmp(fCodecName, "H261") == 0 ||
+	     strcmp(fCodecName, "H263-1998") == 0 || strcmp(fCodecName, "H263-2000") == 0 ||
+	     strcmp(fCodecName, "X-QT") == 0 || strcmp(fCodecName, "X-QUICKTIME") == 0) {
     // This codec requires a specialized RTP payload format; however, we don't yet have an appropriate "RTPSink" subclass for it:
     if (verbosityLevel() > 0) {
       envir() << "\treturns NULL (because we don't have a \"RTPSink\" subclass for this RTP payload format)\n";
@@ -627,12 +689,12 @@ RTPSink* ProxyServerMediaSubsession
     Boolean allowMultipleFramesPerPacket = True; // by default
     Boolean doNormalMBitRule = True; // by default
     // Some codecs change the above default parameters:
-    if (strcmp(codecName, "MP2T") == 0) {
+    if (strcmp(fCodecName, "MP2T") == 0) {
       doNormalMBitRule = False; // no RTP 'M' bit
     }
     newSink = SimpleRTPSink::createNew(envir(), rtpGroupsock,
 				       rtpPayloadTypeIfDynamic, fClientMediaSubsession.rtpTimestampFrequency(),
-				       fClientMediaSubsession.mediumName(), fClientMediaSubsession.codecName(),
+				       fClientMediaSubsession.mediumName(), fCodecName,
 				       fClientMediaSubsession.numChannels(), allowMultipleFramesPerPacket, doNormalMBitRule);
   }
 
@@ -642,11 +704,11 @@ RTPSink* ProxyServerMediaSubsession
 
   // Also tell our "PresentationTimeSubsessionNormalizer" object about the "RTPSink", so it can enable RTCP "SR" reports later:
   PresentationTimeSubsessionNormalizer* ssNormalizer;
-  if (strcmp(codecName, "H264") == 0 ||
-      strcmp(codecName, "H265") == 0 ||
-      strcmp(codecName, "MP4V-ES") == 0 ||
-      strcmp(codecName, "MPV") == 0 ||
-      strcmp(codecName, "DV") == 0) {
+  if (strcmp(fCodecName, "H264") == 0 ||
+      strcmp(fCodecName, "H265") == 0 ||
+      strcmp(fCodecName, "MP4V-ES") == 0 ||
+      strcmp(fCodecName, "MPV") == 0 ||
+      strcmp(fCodecName, "DV") == 0) {
     // There was a separate 'framer' object in front of the "PresentationTimeSubsessionNormalizer", so go back one object to get it:
     ssNormalizer = (PresentationTimeSubsessionNormalizer*)(((FramedFilter*)inputSource)->inputSource());
   } else {
@@ -657,6 +719,18 @@ RTPSink* ProxyServerMediaSubsession
   return newSink;
 }
 
+Groupsock* ProxyServerMediaSubsession::createGroupsock(struct in_addr const& addr, Port port) {
+  ProxyServerMediaSession* parentSession = (ProxyServerMediaSession*)fParentSession;
+  return parentSession->createGroupsock(addr, port);
+}
+
+RTCPInstance* ProxyServerMediaSubsession
+::createRTCP(Groupsock* RTCPgs, unsigned totSessionBW, /* in kbps */
+	     unsigned char const* cname, RTPSink* sink) {
+  ProxyServerMediaSession* parentSession = (ProxyServerMediaSession*)fParentSession;
+  return parentSession->createRTCP(RTCPgs, totSessionBW, cname, sink);
+}
+
 void ProxyServerMediaSubsession::subsessionByeHandler(void* clientData) {
   ((ProxyServerMediaSubsession*)clientData)->subsessionByeHandler();
 }
@@ -668,7 +742,9 @@ void ProxyServerMediaSubsession::subsessionByeHandler() {
 
   // This "BYE" signals that our input source has (effectively) closed, so pass this onto the front-end clients:
   fHaveSetupStream = False; // hack to stop "PAUSE" getting sent by:
-  fClientMediaSubsession.readSource()->handleClosure();
+  if (fClientMediaSubsession.readSource() != NULL) {
+    fClientMediaSubsession.readSource()->handleClosure();
+  }
 
   // And then treat this as if we had lost connection to the back-end server,
   // and can reestablish streaming from it only by sending another "DESCRIBE":
@@ -689,20 +765,21 @@ PresentationTimeSessionNormalizer::PresentationTimeSessionNormalizer(UsageEnviro
 
 PresentationTimeSessionNormalizer::~PresentationTimeSessionNormalizer() {
   while (fSubsessionNormalizers != NULL) {
-    delete fSubsessionNormalizers;
+    Medium::close(fSubsessionNormalizers);
   }
 }
 
-PresentationTimeSubsessionNormalizer*
-PresentationTimeSessionNormalizer::createNewPresentationTimeSubsessionNormalizer(FramedSource* inputSource, RTPSource* rtpSource,
-										 char const* codecName) {
+PresentationTimeSubsessionNormalizer* PresentationTimeSessionNormalizer
+::createNewPresentationTimeSubsessionNormalizer(FramedSource* inputSource, RTPSource* rtpSource,
+						char const* codecName) {
   fSubsessionNormalizers
     = new PresentationTimeSubsessionNormalizer(*this, inputSource, rtpSource, codecName, fSubsessionNormalizers);
   return fSubsessionNormalizers;
 }
 
-void PresentationTimeSessionNormalizer::normalizePresentationTime(PresentationTimeSubsessionNormalizer* ssNormalizer,
-								  struct timeval& toPT, struct timeval const& fromPT) {
+void PresentationTimeSessionNormalizer
+::normalizePresentationTime(PresentationTimeSubsessionNormalizer* ssNormalizer,
+			    struct timeval& toPT, struct timeval const& fromPT) {
   Boolean const hasBeenSynced = ssNormalizer->fRTPSource->hasBeenSynchronizedUsingRTCP();
 
   if (!hasBeenSynced) {
