@@ -11,11 +11,11 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2015, Live Networks, Inc.  All rights reserved
 // A test program that receives a UDP multicast stream
-// and retransmits it to another (multicast or unicast) address & port 
+// and retransmits it to another (multicast or unicast) address & port
 // main program
 
 #include <liveMedia.hh>
@@ -33,7 +33,7 @@ int main(int argc, char** argv) {
   env = BasicUsageEnvironment::createNew(*scheduler);
 
   // Create a 'groupsock' for the input multicast group,port:
-  char* inputAddressStr
+  char const* inputAddressStr
 #ifdef USE_SSM
     = "232.255.42.42";
 #else
@@ -44,7 +44,7 @@ int main(int argc, char** argv) {
 
   Port const inputPort(8888);
   unsigned char const inputTTL = 0; // we're only reading from this mcast group
-  
+
 #ifdef USE_SSM
   char* sourceAddressStr = "aaa.bbb.ccc.ddd";
                            // replace this with the real source address
@@ -55,22 +55,22 @@ int main(int argc, char** argv) {
 #else
   Groupsock inputGroupsock(*env, inputAddress, inputPort, inputTTL);
 #endif
-  
+
   // Then create a liveMedia 'source' object, encapsulating this groupsock:
   FramedSource* source = BasicUDPSource::createNew(*env, &inputGroupsock);
 
 
   // Create a 'groupsock' for the destination address and port:
-  char* outputAddressStr = "239.255.43.43"; // this could also be unicast
+  char const* outputAddressStr = "239.255.43.43"; // this could also be unicast
     // Note: You may change "outputAddressStr" to use a different multicast
     // (or unicast address), but do *not* change it to use the same multicast
     // address as "inputAddressStr".
   struct in_addr outputAddress;
   outputAddress.s_addr = our_inet_addr(outputAddressStr);
-  
+
   Port const outputPort(4444);
   unsigned char const outputTTL = 255;
-  
+
   Groupsock outputGroupsock(*env, outputAddress, outputPort, outputTTL);
 
   // Then create a liveMedia 'sink' object, encapsulating this groupsock:
